/*
Stolyarevskiy Sergey, acobw.narod.ru

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.
	
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
		
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

NDF Version "Samples", 20 april 2007 (prototype NDF version 1.5)
*/

#include <windows.h>
#include <stdio.h>
#include <commctrl.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

inline int R_G_B(int r, int g, int b){return (r<<16 | g<<8 | b);}
inline int GetR(int rgb){return (rgb>>16) & 0xff;}
inline int GetG(int rgb){return (rgb>>8) & 0xff;}
inline int GetB(int rgb){return rgb & 0xff;}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);

typedef struct MyFilterData {
	IFilterPreview		*ifp;
    unsigned char   *old_data;
    long size;
	int NLevel;
	int NLevel_old;
	int Noise;
} MyFilterData;

ScriptFunctionDef _func_defs[]={
    { (ScriptFunctionPtr)ScriptConfig, "Config", "0i" },
    { NULL },
};

CScriptObject ndf_obj={
    NULL, _func_defs
};

struct FilterDefinition filterDef_ = {
    NULL, NULL, NULL,       // next, prev, module
    "NDF Samples",             // name
    "Nonlinear dynamic noise a supressor, version samples",     // desc
    "Stolyarevskiy Sergey", // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size
    InitProc,               // initProc
    NULL,                   // deinitProc
    RunProc,				// runProc
	NULL,				    // paramProc
	ConfigProc,				// configProc
    StringProc,				// stringProc
	StartProc,				// startProc
	EndProc,				// endProc
    &ndf_obj,				// script_obj
    FssProc,				// fssProc
};

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_ = ff->addFilter(fm, &filterDef_, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_);
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
  MyFilterData *mfd = (MyFilterData *)fa->filter_data;
  Pixel32 *src, *dst, *old;
	src=(Pixel32 *)fa->src.data;
    dst=(Pixel32 *)fa->dst.data;
	old=(Pixel32 *)mfd->old_data;
int	th=fa->src.h;
int	tw=fa->src.w;
int i=fa->src.modulo/4;
int L=10-mfd->NLevel;
for (int yt = 0; yt < th; yt++){
for (int xt = 0; xt < tw; xt++){
int	pixel=src[yt*(tw+i)+xt];
int	old_pixel=old[yt*(tw+i)+xt];
int RL=(GetR(pixel)-GetR(old_pixel))*(GetR(pixel)-GetR(old_pixel));
int GL=(GetG(pixel)-GetG(old_pixel))*(GetG(pixel)-GetG(old_pixel));
int BL=(GetB(pixel)-GetB(old_pixel))*(GetB(pixel)-GetB(old_pixel));
int C=2048-L*(RL+GL+BL);
if (C < 0) C = 0;

int corR=C*(GetR(pixel)-GetR(old_pixel));
int corG=C*(GetG(pixel)-GetG(old_pixel));
int corB=C*(GetB(pixel)-GetB(old_pixel));

int CR=GetR(pixel)-(corR>>12);
int CG=GetG(pixel)-(corG>>12);
int CB=GetB(pixel)-(corB>>12);
if (mfd->Noise ){
CR=(GetR(pixel)-CR)*10+127;
CG=(GetG(pixel)-CG)*10+127;
CB=(GetB(pixel)-CB)*10+127;
}
if (CR<0) CR=0; else if (CR>255) CR=255;
if (CG<0) CG=0; else if (CG>255) CG=255;
if (CB<0) CB=0; else if (CB>255) CB=255;

dst[yt*(tw+i)+xt]=R_G_B(CR,CG,CB);
        }
    }
	memcpy (mfd->old_data,fa->dst.data,mfd->size);
		return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
			SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_SETTICFREQ, 1, 0);
            SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_SETRANGE, false, MAKELONG (0, 9));
            SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_SETPOS, true, mfd->NLevel);

			CheckDlgButton(hdlg, IDC_NOISE, mfd->Noise?BST_CHECKED:BST_UNCHECKED);
			mfd->NLevel_old=mfd->NLevel;			
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
            return TRUE;

	case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_LEVEL))
			{
				mfd->NLevel=SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_GETPOS, 0, 0);
				mfd->ifp->RedoFrame();
			}
			break;
     case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
                EndDialog(hdlg, 0);
				return TRUE;
			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				mfd->ifp->RedoFrame();
				break;
			case IDC_NOISE:
				mfd->Noise= !!IsDlgButtonChecked(hdlg, IDC_NOISE);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDCANCEL:
				mfd->NLevel=mfd->NLevel_old;
                EndDialog(hdlg, 1);
                return FALSE;
            }
            break;
    }
    return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;	
	mfd->ifp = fa->ifp;
	return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER), hwnd,
            ConfigDlgProc, (LPARAM)fa->filter_data);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
	 MyFilterData *mfd = (MyFilterData *)fa->filter_data;
		sprintf(str, " ( Level %d )",mfd->NLevel);
	}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->NLevel=argv[0].asInt();
}

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
      MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    _snprintf(buf, buflen, "Config(%d)", mfd->NLevel);
    return true;
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->NLevel=5;
    return 0;
}

int StartProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
mfd->size = fa->src.Size ();
if (!(mfd->old_data = new unsigned char [fa->src.Size()]))
        return 1;
 memset (mfd->old_data, 0, mfd->size);
    return 0;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    delete[] mfd->old_data;
    mfd->old_data = NULL;
    return 0;
}